# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: SignedTimestamp.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='SignedTimestamp.proto',
  package='compubapi_v1',
  syntax='proto3',
  serialized_options=_b('\n$com.yoti.api.client.spi.remote.protoB\024SignedTimestampProtoZ\014yotiprotocom\252\002\031Yoti.Auth.ProtoBuf.Common\312\002\tCompubapi'),
  serialized_pb=_b('\n\x15SignedTimestamp.proto\x12\x0c\x63ompubapi_v1\"\x9b\x01\n\x0fSignedTimestamp\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x11\n\ttimestamp\x18\x02 \x01(\x04\x12\x16\n\x0emessage_digest\x18\x03 \x01(\x0c\x12\x14\n\x0c\x63hain_digest\x18\x04 \x01(\x0c\x12\x1a\n\x12\x63hain_digest_skip1\x18\x05 \x01(\x0c\x12\x1a\n\x12\x63hain_digest_skip2\x18\x06 \x01(\x0c\x42r\n$com.yoti.api.client.spi.remote.protoB\x14SignedTimestampProtoZ\x0cyotiprotocom\xaa\x02\x19Yoti.Auth.ProtoBuf.Common\xca\x02\tCompubapib\x06proto3')
)




_SIGNEDTIMESTAMP = _descriptor.Descriptor(
  name='SignedTimestamp',
  full_name='compubapi_v1.SignedTimestamp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='compubapi_v1.SignedTimestamp.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='compubapi_v1.SignedTimestamp.timestamp', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_digest', full_name='compubapi_v1.SignedTimestamp.message_digest', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chain_digest', full_name='compubapi_v1.SignedTimestamp.chain_digest', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chain_digest_skip1', full_name='compubapi_v1.SignedTimestamp.chain_digest_skip1', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chain_digest_skip2', full_name='compubapi_v1.SignedTimestamp.chain_digest_skip2', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=195,
)

DESCRIPTOR.message_types_by_name['SignedTimestamp'] = _SIGNEDTIMESTAMP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SignedTimestamp = _reflection.GeneratedProtocolMessageType('SignedTimestamp', (_message.Message,), dict(
  DESCRIPTOR = _SIGNEDTIMESTAMP,
  __module__ = 'SignedTimestamp_pb2'
  # @@protoc_insertion_point(class_scope:compubapi_v1.SignedTimestamp)
  ))
_sym_db.RegisterMessage(SignedTimestamp)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import datetime
import logging

import OpenSSL
import asn1
import yoti_python_sdk.protobuf.common_public_api.SignedTimestamp_pb2 as compubapi
from OpenSSL import crypto

from yoti_python_sdk import config

UNKNOWN_EXTENSION = ""
SOURCE_EXTENSION = "1.3.6.1.4.1.47127.1.1.1"
VERIFIER_EXTENSION = "1.3.6.1.4.1.47127.1.1.2"

UNKNOWN_ANCHOR_TYPE = "Unknown"
UNKNOWN_ANCHOR_VALUE = ""


class Anchor:
    def __init__(
        self,
        anchor_type=UNKNOWN_ANCHOR_TYPE,
        sub_type="",
        value="",
        signed_timestamp=None,
        origin_server_certs=None,
    ):
        self.__anchor_type = anchor_type
        self.__sub_type = sub_type
        self.__value = value
        self.__signed_timestamp = signed_timestamp
        self.__origin_server_certs = origin_server_certs

    def __iter__(self):
        return self

    def __next__(self):
        self.idx += 1
        try:
            return self.data[self.idx - 1]
        except IndexError:
            self.idx = 0
            raise StopIteration

    next = __next__  # python2.x compatibility.

    @staticmethod
    def parse_anchors(anchors):
        if anchors is None:
            return None

        parsed_anchors = []
        for anc in anchors:
            if hasattr(anc, "origin_server_certs"):
                anchor_type = UNKNOWN_ANCHOR_TYPE
                try:
                    origin_server_certs_list = list(anc.origin_server_certs)
                    origin_server_certs_item = origin_server_certs_list[0]

                    crypto_cert = crypto.load_certificate(
                        OpenSSL.crypto.FILETYPE_ASN1, origin_server_certs_item
                    ).to_cryptography()

                except Exception as exc:
                    if logging.getLogger().propagate:
                        logging.warning(
                            "Error loading anchor certificate, exception: {0} - {1}".format(
                                type(exc).__name__, exc
                            )
                        )
                    continue

                has_found_anchor = False
                for i in range(len(crypto_cert.extensions)):
                    try:
                        extensions = crypto_cert.extensions[i]
                        if hasattr(extensions, "oid"):
                            oid = extensions.oid
                            if hasattr(oid, "dotted_string"):
                                if oid.dotted_string == SOURCE_EXTENSION:
                                    anchor_type = config.ANCHOR_SOURCE
                                elif oid.dotted_string == VERIFIER_EXTENSION:
                                    anchor_type = config.ANCHOR_VERIFIER

                                if anchor_type != UNKNOWN_ANCHOR_TYPE:
                                    has_found_anchor = True
                                    parsed_anchors = Anchor.get_values_from_extensions(
                                        anc,
                                        anchor_type,
                                        extensions,
                                        crypto_cert,
                                        parsed_anchors,
                                    )

                    except Exception as exc:
                        if logging.getLogger().propagate:
                            logging.warning(
                                "Error parsing anchor certificate extension, exception: {0} - {1}".format(
                                    type(exc).__name__, exc
                                )
                            )
                        continue

                if not has_found_anchor:
                    parsed_anchors.append(
                        Anchor(
                            UNKNOWN_ANCHOR_TYPE,
                            anc.sub_type,
                            UNKNOWN_ANCHOR_VALUE,
                            anc.signed_time_stamp,
                            crypto_cert,
                        )
                    )

        return parsed_anchors

    @staticmethod
    def get_values_from_extensions(
        anc, anchor_type, extensions, crypto_cert, parsed_anchors
    ):
        if hasattr(extensions, "value") and anchor_type != UNKNOWN_ANCHOR_TYPE:
            extension_value = ""
            if hasattr(extensions.value, "value"):
                extension_value = Anchor.decode_asn1_value(extensions.value.value)
            parsed_anchors.append(
                Anchor(
                    anchor_type,
                    anc.sub_type,
                    extension_value,
                    anc.signed_time_stamp,
                    crypto_cert,
                )
            )

        return parsed_anchors

    @staticmethod
    def decode_asn1_value(value_to_decode):
        extension_value_asn1 = value_to_decode
        decoder = asn1.Decoder()

        decoder.start(extension_value_asn1)
        tag, once_decoded_value = decoder.read()

        decoder.start(once_decoded_value)
        tag, twice_decoded_value = decoder.read()

        utf8_value = twice_decoded_value.decode("utf-8")
        return utf8_value

    @property
    def anchor_type(self):
        return self.__anchor_type

    @property
    def value(self):
        return self.__value

    @property
    def sub_type(self):
        return self.__sub_type

    @property
    def signed_timestamp(self):
        if self.__signed_timestamp is None:
            return None

        signed_timestamp_object = compubapi.SignedTimestamp()
        signed_timestamp_object.MergeFromString(self.__signed_timestamp)

        try:
            signed_timestamp_parsed = datetime.datetime.fromtimestamp(
                signed_timestamp_object.timestamp / float(1000000)
            )
        except OSError:
            print(
                "Unable to parse timestamp from integer: '{0}'".format(
                    signed_timestamp_object.timestamp
                )
            )
            return ""

        return signed_timestamp_parsed

    @property
    def origin_server_certs(self):
        return self.__origin_server_certs



# -*- coding: utf-8 -*-

SDK_IDENTIFIER = "Python"
ATTRIBUTE_AGE_OVER = "age_over:"
ATTRIBUTE_AGE_UNDER = "age_under:"
ATTRIBUTE_DATE_OF_BIRTH = "date_of_birth"
ATTRIBUTE_FAMILY_NAME = "family_name"
ATTRIBUTE_FULL_NAME = "full_name"
ATTRIBUTE_GENDER = "gender"
ATTRIBUTE_GIVEN_NAMES = "given_names"
ATTRIBUTE_NATIONALITY = "nationality"
ATTRIBUTE_EMAIL_ADDRESS = "email_address"
ATTRIBUTE_PHONE_NUMBER = "phone_number"
ATTRIBUTE_POSTAL_ADDRESS = "postal_address"
ATTRIBUTE_SELFIE = "selfie"
ATTRIBUTE_STRUCTURED_POSTAL_ADDRESS = "structured_postal_address"
ATTRIBUTE_DOCUMENT_IMAGES = "document_images"
ATTRIBUTE_DOCUMENT_DETAILS = "document_details"
ATTRIBUTE_APPLICATION_NAME = "application_name"
ATTRIBUTE_APPLICATION_LOGO = "application_logo"
ATTRIBUTE_APPLICATION_URL = "application_url"
ATTRIBUTE_APPLICATION_RECEIPT_BGCOLOR = "application_receipt_bgcolor"
ANCHOR_SOURCE = "SOURCE"
ANCHOR_VERIFIER = "VERIFIER"
KEY_AGE_VERIFIED = "is_age_verified"
KEY_BASE64_SELFIE = "base64_selfie_uri"
KEY_FORMATTED_ADDRESS = "formatted_address"
X_YOTI_AUTH_KEY = "X-Yoti-Auth-Key"
X_YOTI_AUTH_DIGEST = "X-Yoti-Auth-Digest"
X_YOTI_SDK = "X-Yoti-SDK"
X_YOTI_SDK_VERSION = X_YOTI_SDK + "-Version"
JSON_CONTENT_TYPE = "application/json"



# -*- coding: utf-8 -*-
import re

from . import date_parser


class DocumentDetails(object):
    VALIDATION_REGEX = re.compile("^[A-Za-z_]* [A-Za-z]{3} [A-Za-z0-9]{1}.*$")

    def __init__(self, data):
        self.__validate_data(data)
        self.__parse_data(data)

    @property
    def document_type(self):
        return self.__document_type

    @property
    def issuing_country(self):
        return self.__issuing_country

    @property
    def document_number(self):
        return self.__document_number

    @property
    def expiration_date(self):
        return self.__dict__.get("_DocumentDetails__expiration_date", None)

    @property
    def issuing_authority(self):
        return self.__dict__.get("_DocumentDetails__issuing_authority", None)

    def __validate_data(self, data):
        if self.VALIDATION_REGEX.search(data):
            return
        else:
            raise ValueError("Invalid value for DocumentDetails")

    def __parse_data(self, data):
        data = data.split()

        self.__document_type = data[0]
        self.__issuing_country = data[1]
        self.__document_number = data[2]
        if len(data) > 3:
            date = data[3]
            if date != "-":
                self.__expiration_date = date_parser.from_iso_format(date)
        if len(data) > 4:
            self.__issuing_authority = data[4]



# -*- coding: utf-8 -*-
from cryptography.fernet import hashes, base64
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


class Crypto:
    def __init__(self, pem_container):
        self.private_key = serialization.load_pem_private_key(
            data=pem_container, password=None, backend=default_backend()
        )

    def get_public_key(self):
        public_key = self.private_key.public_key()
        der = public_key.public_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
        return base64.b64encode(der).decode("utf-8")

    def decrypt_token(self, encrypted_token):
        try:
            if not isinstance(encrypted_token, bytes):
                # On Python 2 the token is str and to b64decode it we need bytes or unicode
                encrypted_token = encrypted_token.encode("utf-8")

            data = base64.urlsafe_b64decode(encrypted_token)
            decrypted = self.private_key.decrypt(
                ciphertext=data, padding=padding.PKCS1v15()
            )
            return decrypted

        except Exception as exc:
            raise ValueError(
                "Could not decrypt token: {0}, {1}".format(encrypted_token, exc)
            )

    def sign(self, message):
        signature = self.private_key.sign(
            data=message.encode("utf-8"),
            padding=padding.PKCS1v15(),
            algorithm=hashes.SHA256(),
        )
        return base64.b64encode(signature).decode("utf-8")

    @staticmethod
    def decipher(key, iv, cipher_text):
        decryptor = Cipher(
            algorithm=algorithms.AES(key), mode=modes.CBC(iv), backend=default_backend()
        ).decryptor()
        plaintext = decryptor.update(cipher_text) + decryptor.finalize()

        return Crypto.strip_pkcs5_padding(plaintext)

    @staticmethod
    def strip_pkcs5_padding(data):
        if isinstance(data, str):
            data = bytearray(data)

        number_of_padded_bytes = data[-1]
        stripped = data[:-number_of_padded_bytes]

        if isinstance(stripped, bytearray):
            stripped = str(stripped)
        return stripped



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class LocationConstraintExtensionBuilder(object):
    LOCATION_CONSTRAINT = "LOCATION_CONSTRAINT"

    def __init__(self):
        self.__extension = {}
        self.__extension["type"] = self.LOCATION_CONSTRAINT
        self.__extension["content"] = {}
        self.__device_location = {
            "latitude": None,
            "longtitude": None,
            "radius": None,
            "max_uncertainty_radius": None,
        }
        self.__extension["content"]["expected_device_location"] = self.__device_location

    def with_latitude(self, latitude):
        self.__device_location["latitude"] = latitude
        return self

    def with_longtitude(self, longtitude):
        self.__device_location["longtitude"] = longtitude
        return self

    def with_radius(self, radius):
        self.__device_location["radius"] = radius
        return self

    def with_uncertainty(self, uncertainty):
        self.__device_location["max_uncertainty_radius"] = uncertainty
        return self

    def build(self):
        return self.__extension.copy()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .policy.dynamic_policy_builder import DynamicPolicyBuilder


class DynamicScenarioBuilder(object):
    def __init__(self):
        self.__scenario = {
            "policy": DynamicPolicyBuilder().build(),
            "extensions": [],
            "callback_endpoint": "",
        }

    """
    @param policy A DynamicPolicy defining the attributes to be shared
    """

    def with_policy(self, policy):
        self.__scenario["policy"] = policy
        return self

    """
    @param extension An extension to be activated for the scenario
    """

    def with_extension(self, extension):
        self.__scenario["extensions"].append(extension)
        return self

    """
    @param callback_endpoint A string with the callback endpoint
    """

    def with_callback_endpoint(self, callback_endpoint):
        self.__scenario["callback_endpoint"] = callback_endpoint
        return self

    """
    @return Dictionary representation of dynamic scenario
    """

    def build(self):
        return self.__scenario.copy()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: List.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Attribute_pb2 as Attribute__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='List.proto',
  package='attrpubapi_v1',
  syntax='proto3',
  serialized_options=_b('\n$com.yoti.api.client.spi.remote.protoB\022AttributeListProtoZ\ryotiprotoattr\252\002\034Yoti.Auth.ProtoBuf.Attribute\312\002\nAttrpubapi'),
  serialized_pb=_b('\n\nList.proto\x12\rattrpubapi_v1\x1a\x0f\x41ttribute.proto\"S\n\x0e\x41ttributeAndId\x12+\n\tattribute\x18\x01 \x01(\x0b\x32\x18.attrpubapi_v1.Attribute\x12\x14\n\x0c\x61ttribute_id\x18\x02 \x01(\x0c\"R\n\x12\x41ttributeAndIdList\x12<\n\x15\x61ttribute_and_id_list\x18\x01 \x03(\x0b\x32\x1d.attrpubapi_v1.AttributeAndId\"=\n\rAttributeList\x12,\n\nattributes\x18\x01 \x03(\x0b\x32\x18.attrpubapi_v1.AttributeBu\n$com.yoti.api.client.spi.remote.protoB\x12\x41ttributeListProtoZ\ryotiprotoattr\xaa\x02\x1cYoti.Auth.ProtoBuf.Attribute\xca\x02\nAttrpubapib\x06proto3')
  ,
  dependencies=[Attribute__pb2.DESCRIPTOR,])




_ATTRIBUTEANDID = _descriptor.Descriptor(
  name='AttributeAndId',
  full_name='attrpubapi_v1.AttributeAndId',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute', full_name='attrpubapi_v1.AttributeAndId.attribute', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_id', full_name='attrpubapi_v1.AttributeAndId.attribute_id', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=129,
)


_ATTRIBUTEANDIDLIST = _descriptor.Descriptor(
  name='AttributeAndIdList',
  full_name='attrpubapi_v1.AttributeAndIdList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attribute_and_id_list', full_name='attrpubapi_v1.AttributeAndIdList.attribute_and_id_list', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=131,
  serialized_end=213,
)


_ATTRIBUTELIST = _descriptor.Descriptor(
  name='AttributeList',
  full_name='attrpubapi_v1.AttributeList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributes', full_name='attrpubapi_v1.AttributeList.attributes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=276,
)

_ATTRIBUTEANDID.fields_by_name['attribute'].message_type = Attribute__pb2._ATTRIBUTE
_ATTRIBUTEANDIDLIST.fields_by_name['attribute_and_id_list'].message_type = _ATTRIBUTEANDID
_ATTRIBUTELIST.fields_by_name['attributes'].message_type = Attribute__pb2._ATTRIBUTE
DESCRIPTOR.message_types_by_name['AttributeAndId'] = _ATTRIBUTEANDID
DESCRIPTOR.message_types_by_name['AttributeAndIdList'] = _ATTRIBUTEANDIDLIST
DESCRIPTOR.message_types_by_name['AttributeList'] = _ATTRIBUTELIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AttributeAndId = _reflection.GeneratedProtocolMessageType('AttributeAndId', (_message.Message,), dict(
  DESCRIPTOR = _ATTRIBUTEANDID,
  __module__ = 'List_pb2'
  # @@protoc_insertion_point(class_scope:attrpubapi_v1.AttributeAndId)
  ))
_sym_db.RegisterMessage(AttributeAndId)

AttributeAndIdList = _reflection.GeneratedProtocolMessageType('AttributeAndIdList', (_message.Message,), dict(
  DESCRIPTOR = _ATTRIBUTEANDIDLIST,
  __module__ = 'List_pb2'
  # @@protoc_insertion_point(class_scope:attrpubapi_v1.AttributeAndIdList)
  ))
_sym_db.RegisterMessage(AttributeAndIdList)

AttributeList = _reflection.GeneratedProtocolMessageType('AttributeList', (_message.Message,), dict(
  DESCRIPTOR = _ATTRIBUTELIST,
  __module__ = 'List_pb2'
  # @@protoc_insertion_point(class_scope:attrpubapi_v1.AttributeList)
  ))
_sym_db.RegisterMessage(AttributeList)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-

import datetime


def from_iso_format(string):
    parts = [int(a) for a in string.split("-")]

    if len(parts) != 3:
        raise ValueError

    return datetime.date(parts[0], parts[1], parts[2])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from cryptography.fernet import base64
from yoti_python_sdk.protobuf.attribute_public_api import Attribute_pb2, List_pb2
from yoti_python_sdk.protobuf.common_public_api import EncryptedData_pb2


class Protobuf(object):
    CT_UNDEFINED = 0  # should not be seen, and is used as an error placeholder
    CT_STRING = 1  # UTF-8 encoded text
    CT_JPEG = 2  # standard .jpeg image
    CT_DATE = 3  # string in RFC3339 format (YYYY-MM-DD)
    CT_PNG = 4  # standard .png image
    CT_JSON = 5  # value encoded using JSON
    CT_MULTI_VALUE = 6  # allows a list of values
    CT_INT = 7  # int value

    @staticmethod
    def current_user(receipt):
        if receipt.get('other_party_profile_content') is None or receipt.get('other_party_profile_content') == '':
            return None

        profile_content = receipt['other_party_profile_content']
        decoded_profile_content = base64.b64decode(profile_content)

        merged_user = EncryptedData_pb2.EncryptedData()
        merged_user.MergeFromString(decoded_profile_content)
        return merged_user

    @staticmethod
    def current_application(receipt):
        if receipt.get("profile_content") is None or receipt.get("profile_content") == '':
            return None

        application_content = receipt["profile_content"]
        decoded_application_content = base64.b64decode(application_content)

        merged_application = EncryptedData_pb2.EncryptedData()
        merged_application.MergeFromString(decoded_application_content)

        return merged_application

    @staticmethod
    def attribute_list(data):
        attribute_list = List_pb2.AttributeList()
        attribute_list.MergeFromString(data)
        return attribute_list

    @staticmethod
    def anchor(data):
        anchor = Attribute_pb2.Anchor()
        anchor.MergeFromString(data)
        return anchor

    @staticmethod
    def attribute(data):
        attribute = Attribute_pb2.Attribute()
        attribute.MergeFromString(data)
        return attribute

    @staticmethod
    def multi_value(data):
        multi_value = Attribute_pb2.MultiValue()
        multi_value.MergeFromString(data)
        return multi_value



# -*- coding: utf-8 -*-
import os
from distutils.util import convert_path
from os import environ

from yoti_python_sdk.client import Client

DEFAULTS = {
    "YOTI_API_URL": "https://api.yoti.com",
    "YOTI_API_PORT": 443,
    "YOTI_API_VERSION": "v1",
    "YOTI_API_VERIFY_SSL": "true"
}

main_ns = {}

directory_name = os.path.dirname(__file__)
version_path = os.path.join(directory_name, "version.py")

ver_path = convert_path(version_path)
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

__version__ = main_ns["__version__"]
YOTI_API_URL = environ.get("YOTI_API_URL", DEFAULTS["YOTI_API_URL"])
YOTI_API_PORT = environ.get("YOTI_API_PORT", DEFAULTS["YOTI_API_PORT"])
YOTI_API_VERSION = environ.get("YOTI_API_VERSION", DEFAULTS["YOTI_API_VERSION"])
YOTI_API_ENDPOINT = environ.get("YOTI_API_ENDPOINT", "{0}:{1}/api/{2}".format(
    YOTI_API_URL, YOTI_API_PORT, YOTI_API_VERSION
))

YOTI_API_VERIFY_SSL = environ.get("YOTI_API_VERIFY_SSL", DEFAULTS["YOTI_API_VERIFY_SSL"])
if YOTI_API_VERIFY_SSL.lower() == "false":
    YOTI_API_VERIFY_SSL = False
else:
    YOTI_API_VERIFY_SSL = True

__all__ = [
    "Client",
    __version__
]



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Signing.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ContentType_pb2 as ContentType__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Signing.proto',
  package='attrpubapi_v1',
  syntax='proto3',
  serialized_options=_b('\n$com.yoti.api.client.spi.remote.protoB\014SigningProtoZ\ryotiprotoattr\252\002\034Yoti.Auth.ProtoBuf.Attribute\312\002\nAttrpubapi'),
  serialized_pb=_b('\n\rSigning.proto\x12\rattrpubapi_v1\x1a\x11\x43ontentType.proto\"\xaa\x01\n\x10\x41ttributeSigning\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x30\n\x0c\x63ontent_type\x18\x03 \x01(\x0e\x32\x1a.attrpubapi_v1.ContentType\x12\x1a\n\x12\x61rtifact_signature\x18\x04 \x01(\x0c\x12\x10\n\x08sub_type\x18\x05 \x01(\t\x12\x19\n\x11signed_time_stamp\x18\x06 \x01(\x0c\x42o\n$com.yoti.api.client.spi.remote.protoB\x0cSigningProtoZ\ryotiprotoattr\xaa\x02\x1cYoti.Auth.ProtoBuf.Attribute\xca\x02\nAttrpubapib\x06proto3')
  ,
  dependencies=[ContentType__pb2.DESCRIPTOR,])




_ATTRIBUTESIGNING = _descriptor.Descriptor(
  name='AttributeSigning',
  full_name='attrpubapi_v1.AttributeSigning',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='attrpubapi_v1.AttributeSigning.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='attrpubapi_v1.AttributeSigning.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='content_type', full_name='attrpubapi_v1.AttributeSigning.content_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='artifact_signature', full_name='attrpubapi_v1.AttributeSigning.artifact_signature', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sub_type', full_name='attrpubapi_v1.AttributeSigning.sub_type', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signed_time_stamp', full_name='attrpubapi_v1.AttributeSigning.signed_time_stamp', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=222,
)

_ATTRIBUTESIGNING.fields_by_name['content_type'].enum_type = ContentType__pb2._CONTENTTYPE
DESCRIPTOR.message_types_by_name['AttributeSigning'] = _ATTRIBUTESIGNING
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AttributeSigning = _reflection.GeneratedProtocolMessageType('AttributeSigning', (_message.Message,), dict(
  DESCRIPTOR = _ATTRIBUTESIGNING,
  __module__ = 'Signing_pb2'
  # @@protoc_insertion_point(class_scope:attrpubapi_v1.AttributeSigning)
  ))
_sym_db.RegisterMessage(AttributeSigning)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import time
import uuid


class Endpoint(object):
    def __init__(self, sdk_id):
        self.sdk_id = sdk_id

    def get_activity_details_request_path(self, decrypted_request_token):
        return "/profile/{0}?nonce={1}&timestamp={2}&appId={3}".format(
            decrypted_request_token,
            self.__create_nonce(),
            self.__create_timestamp(),
            self.sdk_id,
        )

    def get_aml_request_url(self):
        return "/aml-check?appId={0}&timestamp={1}&nonce={2}".format(
            self.sdk_id, self.__create_timestamp(), self.__create_nonce()
        )

    def get_dynamic_share_request_url(self):
        return "/qrcodes/apps/{appid}?nonce={nonce}&timestamp={timestamp}".format(
            appid=self.sdk_id,
            nonce=self.__create_nonce(),
            timestamp=self.__create_timestamp(),
        )

    @staticmethod
    def __create_nonce():
        return uuid.uuid4()

    @staticmethod
    def __create_timestamp():
        return int(time.time() * 1000)



import json


class AmlResult:
    def __init__(self, response_text):
        if not response_text:
            raise ValueError("AML Response is not valid")

        try:
            self.on_pep_list = json.loads(response_text).get("on_pep_list")
            self.on_fraud_list = json.loads(response_text).get("on_fraud_list")
            self.on_watch_list = json.loads(response_text).get("on_watch_list")

        except (AttributeError, IOError, TypeError, OSError) as exc:
            error = 'Could not parse AML result from response: "{0}"'.format(
                response_text
            )
            exception = "{0}: {1}".format(type(exc).__name__, exc)
            raise RuntimeError("{0}: {1}".format(error, exception))

        self.__check_for_none_values(self.on_pep_list)
        self.__check_for_none_values(self.on_fraud_list)
        self.__check_for_none_values(self.on_watch_list)

    @staticmethod
    def __check_for_none_values(arg):
        if arg is None:
            raise TypeError(
                str.format(
                    "{0} argument was unable to be retrieved from the response", arg
                )
            )

    def __iter__(self):
        yield "on_pep_list", self.on_pep_list
        yield "on_fraud_list", self.on_fraud_list
        yield "on_watch_list", self.on_watch_list


class AmlAddress:
    def __init__(self, country, postcode=None):
        self.country = country
        self.post_code = postcode


class AmlProfile:
    def __init__(self, given_names, family_name, address, ssn=None):
        self.given_names = given_names
        self.family_name = family_name
        self.address = address.__dict__
        self.ssn = ssn



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import collections
import json
import logging

from yoti_python_sdk import multivalue
from yoti_python_sdk.protobuf.protobuf import Protobuf


def value_based_on_content_type(value, content_type=None):
    from yoti_python_sdk.image import Image

    if content_type == Protobuf.CT_STRING:
        return value.decode("utf-8")
    elif value == b"":
        raise ValueError(
            "Content type: '{0}' should not have an empty value".format(content_type)
        )
    elif content_type == Protobuf.CT_DATE:
        return value.decode("utf-8")
    elif content_type in Image.allowed_types():
        return Image(value, content_type)
    elif content_type == Protobuf.CT_JSON:
        return convert_to_dict(value)
    elif content_type == Protobuf.CT_INT:
        string_value = value.decode("utf-8")
        int_value = int(string_value)
        return int_value
    elif content_type == Protobuf.CT_MULTI_VALUE:
        return tuple(multivalue.parse(value))

    if logging.getLogger().propagate:
        logging.warning(
            "Unknown type '{0}', attempting to parse it as a String".format(
                content_type
            )
        )

    return value.decode("utf-8")


def convert_to_dict(byte_value):
    decoder = json.JSONDecoder(object_pairs_hook=collections.OrderedDict, strict=False)
    value_to_decode = byte_value.decode()

    return decoder.decode(value_to_decode)



# -*- coding: utf-8 -*-

from .dynamic_policy_builder import DynamicPolicyBuilder
from .wanted_attribute_builder import WantedAttributeBuilder

__all__ = ["DynamicPolicyBuilder", "WantedAttributeBuilder"]



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

from yoti_python_sdk import __version__

long_description = (
    "This package contains the tools you need to quickly "
    "integrate your Python back-end with Yoti, so that your "
    "users can share their identity details with your "
    "application in a secure and trusted way."
)

setup(
    name="yoti",
    version=__version__,
    packages=find_packages(),
    license="MIT",
    description="The Yoti Python SDK, providing API support for Login, Verify (2FA) and Age Verification.",
    long_description=long_description,
    url="https://github.com/getyoti/yoti-python-sdk",
    author="Yoti",
    author_email="websdk@yoti.com",
    install_requires=[
        "cryptography>=2.2.1",
        "protobuf>=3.1.0",
        "requests>=2.11.1",
        "future>=0.11.0",
        "asn1==2.2.0",
        "pyopenssl>=18.0.0",
    ],
    extras_require={
        "examples": [
            "Django>1.11.16",
            "Flask>=0.10",
            "python-dotenv>=0.7.1",
            "django-sslserver>=0.2",
            "Werkzeug==0.11.15",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="yoti sdk 2FA multifactor authentication verification identity login register verify 2Factor",
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: EncryptedData.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='EncryptedData.proto',
  package='compubapi_v1',
  syntax='proto3',
  serialized_options=_b('\n$com.yoti.api.client.spi.remote.protoB\022EncryptedDataProtoZ\014yotiprotocom\252\002\031Yoti.Auth.ProtoBuf.Common\312\002\tCompubapi'),
  serialized_pb=_b('\n\x13\x45ncryptedData.proto\x12\x0c\x63ompubapi_v1\"0\n\rEncryptedData\x12\n\n\x02iv\x18\x01 \x01(\x0c\x12\x13\n\x0b\x63ipher_text\x18\x02 \x01(\x0c\x42p\n$com.yoti.api.client.spi.remote.protoB\x12\x45ncryptedDataProtoZ\x0cyotiprotocom\xaa\x02\x19Yoti.Auth.ProtoBuf.Common\xca\x02\tCompubapib\x06proto3')
)




_ENCRYPTEDDATA = _descriptor.Descriptor(
  name='EncryptedData',
  full_name='compubapi_v1.EncryptedData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='iv', full_name='compubapi_v1.EncryptedData.iv', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cipher_text', full_name='compubapi_v1.EncryptedData.cipher_text', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=37,
  serialized_end=85,
)

DESCRIPTOR.message_types_by_name['EncryptedData'] = _ENCRYPTEDDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EncryptedData = _reflection.GeneratedProtocolMessageType('EncryptedData', (_message.Message,), dict(
  DESCRIPTOR = _ENCRYPTEDDATA,
  __module__ = 'EncryptedData_pb2'
  # @@protoc_insertion_point(class_scope:compubapi_v1.EncryptedData)
  ))
_sym_db.RegisterMessage(EncryptedData)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

"""
Builder for WantedAttribute
"""


class WantedAttributeBuilder(object):
    def __init__(self):
        self.__attribute = {}

    def with_name(self, name):
        """
        :param name: Sets name
        """
        self.__attribute["name"] = name
        return self

    def with_derivation(self, derivation):
        """
        :param derivation: Sets derivation
        """
        self.__attribute["derivation"] = derivation
        return self

    def build(self):
        """
        :return: The wanted attribute object
        """
        return self.__attribute.copy()



# -*- coding: utf-8 -*-
import collections
import json
import logging
from deprecated import deprecated
from datetime import datetime

from yoti_python_sdk import attribute_parser, config
from yoti_python_sdk.profile import Profile, ApplicationProfile


class ActivityDetails:
    def __init__(self, receipt, decrypted_profile=None, decrypted_application_profile=None):
        self.decrypted_profile = decrypted_profile
        self.user_profile = {}  # will be removed in v3.0.0
        self.base64_selfie_uri = None
        self.decrypted_application_profile = decrypted_application_profile

        if decrypted_profile and hasattr(decrypted_profile, "attributes"):
            decrypted_profile_attributes = decrypted_profile.attributes
            self.profile = Profile(decrypted_profile_attributes)

            for field in decrypted_profile_attributes:  # will be removed in v3.0.0
                try:
                    value = attribute_parser.value_based_on_content_type(
                        field.value, field.content_type
                    )

                    if field.name == config.ATTRIBUTE_SELFIE:
                        self.base64_selfie_uri = value.base64_content()
                        value = (
                            field.value
                        )  # set value to be byte content, for backwards compatibility

                    if field.name == config.ATTRIBUTE_STRUCTURED_POSTAL_ADDRESS:
                        value = self.try_convert_structured_postal_address_to_dict(
                            field
                        )

                    if field.name.startswith(
                        config.ATTRIBUTE_AGE_OVER
                    ) or field.name.startswith(config.ATTRIBUTE_AGE_UNDER):
                        self.try_parse_age_verified_field(field)

                    self.user_profile[field.name] = value

                except ValueError as ve:
                    if logging.getLogger().propagate:
                        logging.warning(ve)
                except Exception as exc:
                    if logging.getLogger().propagate:
                        logging.warning(
                            "Error parsing profile attribute:{0}, exception: {1} - {2}".format(
                                field.name, type(exc).__name__, exc
                            )
                        )

            self.ensure_postal_address()

        if decrypted_application_profile and hasattr(decrypted_application_profile, "attributes"):
            decrypted_application_profile_attributes = decrypted_application_profile.attributes
            self.application_profile = ApplicationProfile(decrypted_application_profile_attributes)

        self.__remember_me_id = receipt.get("remember_me_id")
        self.parent_remember_me_id = receipt.get("parent_remember_me_id")
        self.outcome = receipt.get("sharing_outcome")
        self.receipt_id = receipt.get("receipt_id")
        timestamp = receipt.get("timestamp")

        if timestamp is not None:
            self.timestamp = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%SZ")

    @property
    def remember_me_id(self):
        return self.__remember_me_id

    @property
    @deprecated
    def user_id(self):
        return self.__remember_me_id

    @user_id.setter
    @deprecated
    def user_id(self, value):
        self.__remember_me_id = value

    def try_parse_age_verified_field(self, field):
        if field is not None:
            age_verified = attribute_parser.value_based_on_content_type(
                field.value, field.content_type
            )
            if age_verified == "true":
                self.user_profile[config.KEY_AGE_VERIFIED] = True
                return
            if age_verified == "false":
                self.user_profile[config.KEY_AGE_VERIFIED] = False
                return

            print(
                "age_verified_field value: '{0}' was unable to be parsed into a boolean value".format(
                    age_verified
                )
            )

    @staticmethod
    def try_convert_structured_postal_address_to_dict(field):
        decoder = json.JSONDecoder(
            object_pairs_hook=collections.OrderedDict, strict=False
        )
        value_to_decode = field.value
        if not isinstance(value_to_decode, str):
            value_to_decode = value_to_decode.decode()

        return decoder.decode(value_to_decode)

    def ensure_postal_address(self):
        # setting in 'user_profile'  - will be removed once user_profile is removed
        if (
            config.ATTRIBUTE_POSTAL_ADDRESS not in self.user_profile
            and config.ATTRIBUTE_STRUCTURED_POSTAL_ADDRESS in self.user_profile
        ):
            if (
                config.KEY_FORMATTED_ADDRESS
                in self.user_profile[config.ATTRIBUTE_STRUCTURED_POSTAL_ADDRESS]
            ):
                self.user_profile[config.ATTRIBUTE_POSTAL_ADDRESS] = self.user_profile[
                    config.ATTRIBUTE_STRUCTURED_POSTAL_ADDRESS
                ][config.KEY_FORMATTED_ADDRESS]

    def __iter__(self):
        yield "user_id", self.__remember_me_id  # Using the private member directly to avoid a deprecation warning
        yield "parent_remember_me_id", self.parent_remember_me_id
        yield "outcome", self.outcome
        yield "receipt_id", self.receipt_id
        yield "user_profile", self.user_profile
        yield "profile", self.profile
        yield "base64_selfie_uri", self.base64_selfie_uri
        yield "remember_me_id", self.remember_me_id



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ContentType.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='ContentType.proto',
  package='attrpubapi_v1',
  syntax='proto3',
  serialized_options=_b('\n$com.yoti.api.client.spi.remote.protoB\020ContentTypeProtoZ\ryotiprotoattr\252\002\034Yoti.Auth.ProtoBuf.Attribute\312\002\nAttrpubapi'),
  serialized_pb=_b('\n\x11\x43ontentType.proto\x12\rattrpubapi_v1*i\n\x0b\x43ontentType\x12\r\n\tUNDEFINED\x10\x00\x12\n\n\x06STRING\x10\x01\x12\x08\n\x04JPEG\x10\x02\x12\x08\n\x04\x44\x41TE\x10\x03\x12\x07\n\x03PNG\x10\x04\x12\x08\n\x04JSON\x10\x05\x12\x0f\n\x0bMULTI_VALUE\x10\x06\x12\x07\n\x03INT\x10\x07\x42s\n$com.yoti.api.client.spi.remote.protoB\x10\x43ontentTypeProtoZ\ryotiprotoattr\xaa\x02\x1cYoti.Auth.ProtoBuf.Attribute\xca\x02\nAttrpubapib\x06proto3')
)

_CONTENTTYPE = _descriptor.EnumDescriptor(
  name='ContentType',
  full_name='attrpubapi_v1.ContentType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNDEFINED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STRING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JPEG', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DATE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PNG', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JSON', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MULTI_VALUE', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INT', index=7, number=7,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=36,
  serialized_end=141,
)
_sym_db.RegisterEnumDescriptor(_CONTENTTYPE)

ContentType = enum_type_wrapper.EnumTypeWrapper(_CONTENTTYPE)
UNDEFINED = 0
STRING = 1
JPEG = 2
DATE = 3
PNG = 4
JSON = 5
MULTI_VALUE = 6
INT = 7


DESCRIPTOR.enum_types_by_name['ContentType'] = _CONTENTTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
from os import environ
from os.path import isfile, expanduser

import requests
from cryptography.fernet import base64
from past.builtins import basestring

import yoti_python_sdk
from yoti_python_sdk import aml
from yoti_python_sdk.activity_details import ActivityDetails
from yoti_python_sdk.crypto import Crypto
from yoti_python_sdk.endpoint import Endpoint
from yoti_python_sdk.protobuf import protobuf
from .config import (
    X_YOTI_AUTH_KEY,
    X_YOTI_AUTH_DIGEST,
    X_YOTI_SDK,
    SDK_IDENTIFIER,
    X_YOTI_SDK_VERSION,
    JSON_CONTENT_TYPE,
)

NO_KEY_FILE_SPECIFIED_ERROR = (
    "Please specify the correct private key file "
    "in Client(pem_file_path=...)\nor by setting "
    'the "YOTI_KEY_FILE_PATH" environment variable'
)
HTTP_SUPPORTED_METHODS = ["POST", "PUT", "PATCH", "GET", "DELETE"]

UNKNOWN_HTTP_ERROR = "Unknown HTTP error occured: {0} {1}"
DEFAULT_HTTP_CLIENT_ERRORS = {"default": UNKNOWN_HTTP_ERROR}


class Client(object):
    def __init__(self, sdk_id=None, pem_file_path=None):
        self.sdk_id = sdk_id or environ.get("YOTI_CLIENT_SDK_ID")
        pem_file_path_env = environ.get("YOTI_KEY_FILE_PATH")

        if pem_file_path is not None:
            error_source = "argument specified in Client()"
            pem = self.__read_pem_file(pem_file_path, error_source)
        elif pem_file_path_env is not None:
            error_source = "specified by the YOTI_KEY_FILE_PATH env variable"
            pem = self.__read_pem_file(pem_file_path_env, error_source)
        else:
            raise RuntimeError(NO_KEY_FILE_SPECIFIED_ERROR)

        self.__crypto = Crypto(pem)
        self.__endpoint = Endpoint(sdk_id)

    @staticmethod
    def __read_pem_file(key_file_path, error_source):
        try:
            key_file_path = expanduser(key_file_path)

            if not isinstance(key_file_path, basestring) or not isfile(key_file_path):
                raise IOError("File not found: {0}".format(key_file_path))
            with open(key_file_path, "rb") as pem_file:
                return pem_file.read().strip()
        except (AttributeError, IOError, TypeError, OSError) as exc:
            error = 'Could not read private key file: "{0}", passed as: {1} '.format(
                key_file_path, error_source
            )
            exception = "{0}: {1}".format(type(exc).__name__, exc)
            raise RuntimeError("{0}: {1}".format(error, exception))

    def get_activity_details(self, encrypted_request_token):
        proto = protobuf.Protobuf()
        http_method = "GET"
        content = None
        response = self.__make_activity_details_request(
            encrypted_request_token, http_method, content
        )
        receipt = json.loads(response.text).get("receipt")

        encrypted_data = proto.current_user(receipt)
        encrypted_application_profile = proto.current_application(receipt)

        if not encrypted_data:
            return ActivityDetails(receipt)

        unwrapped_key = self.__crypto.decrypt_token(receipt["wrapped_receipt_key"])

        decrypted_profile_data = self.__crypto.decipher(
            unwrapped_key, encrypted_data.iv, encrypted_data.cipher_text
        )
        decrypted_application_data = self.__crypto.decipher(
            unwrapped_key, encrypted_application_profile.iv, encrypted_application_profile.cipher_text
        )

        user_profile_attribute_list = proto.attribute_list(decrypted_profile_data)
        application_profile_attribute_list = proto.attribute_list(decrypted_application_data)

        return ActivityDetails(
            receipt=receipt, decrypted_profile=user_profile_attribute_list, decrypted_application_profile=application_profile_attribute_list
        )

    def perform_aml_check(self, aml_profile):
        if aml_profile is None:
            raise TypeError("aml_profile not set")

        http_method = "POST"

        response = self.__make_aml_check_request(http_method, aml_profile)

        return aml.AmlResult(response.text)

    def make_request(self, http_method, endpoint, body):
        url = yoti_python_sdk.YOTI_API_ENDPOINT + endpoint
        headers = self.__get_request_headers(endpoint, http_method, body)
        response = requests.request(http_method, url, headers=headers, data=body, verify=yoti_python_sdk.YOTI_API_VERIFY_SSL)
        return response

    @property
    def endpoints(self):
        return self.__endpoint

    @staticmethod
    def http_error_handler(response, error_messages={}):
        status_code = response.status_code
        if 200 <= status_code < 300:
            return
        elif status_code in error_messages.keys():
            raise RuntimeError(
                error_messages[status_code].format(status_code, response.text)
            )
        elif "default" in error_messages.keys():
            raise RuntimeError(
                error_messages["default"].format(status_code, response.text)
            )
        else:
            raise RuntimeError(UNKNOWN_HTTP_ERROR.format(status_code, response.text))

    def __make_activity_details_request(
        self, encrypted_request_token, http_method, content
    ):
        decrypted_token = self.__crypto.decrypt_token(encrypted_request_token).decode(
            "utf-8"
        )
        path = self.__endpoint.get_activity_details_request_path(decrypted_token)
        url = yoti_python_sdk.YOTI_API_ENDPOINT + path
        headers = self.__get_request_headers(path, http_method, content)
        response = requests.get(url=url, headers=headers, verify=yoti_python_sdk.YOTI_API_VERIFY_SSL)

        self.http_error_handler(
            response, {"default": "Unsuccessful Yoti API call: {1}"}
        )

        return response

    def __make_aml_check_request(self, http_method, aml_profile):
        aml_profile_json = json.dumps(aml_profile.__dict__, sort_keys=True)
        aml_profile_bytes = aml_profile_json.encode()
        path = self.__endpoint.get_aml_request_url()
        url = yoti_python_sdk.YOTI_API_ENDPOINT + path
        headers = self.__get_request_headers(path, http_method, aml_profile_bytes)

        response = requests.post(url=url, headers=headers, data=aml_profile_bytes, verify=yoti_python_sdk.YOTI_API_VERIFY_SSL)

        self.http_error_handler(
            response, {"default": "Unsuccessful Yoti API call: {1}"}
        )

        return response

    def __get_request_headers(self, path, http_method, content):
        request = self.__create_request(http_method, path, content)
        sdk_version = yoti_python_sdk.__version__

        return {
            X_YOTI_AUTH_KEY: self.__crypto.get_public_key(),
            X_YOTI_AUTH_DIGEST: self.__crypto.sign(request),
            X_YOTI_SDK: SDK_IDENTIFIER,
            X_YOTI_SDK_VERSION: "{0}-{1}".format(SDK_IDENTIFIER, sdk_version),
            "Content-Type": JSON_CONTENT_TYPE,
            "Accept": JSON_CONTENT_TYPE,
        }

    @staticmethod
    def __create_request(http_method, path, content):
        if http_method not in HTTP_SUPPORTED_METHODS:
            raise ValueError(
                "{} is not in the list of supported methods: {}".format(
                    http_method, HTTP_SUPPORTED_METHODS
                )
            )

        request = "{}&{}".format(http_method, path)

        if content is not None:
            b64encoded = base64.b64encode(content)
            b64ascii = b64encoded.decode("ascii")
            request += "&" + b64ascii

        return request
